import easydev


def test_path():
    p = easydev.get_path_sphinx_themes()
    import os

    dirs = os.listdir(p)
    assert "standard" in dirs


def test_sphinx_themes():
    p = easydev.get_sphinx_themes()
    assert "standard" in p



# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of the easydev software
#
#  Copyright (c) 2011-2024
#
#  File author(s): Thomas Cokelaer <cokelaer@gmail.com>
#
#  Distributed under the BSD3 License.
#
#  Website: https://github.com/cokelaer/easydev
#  Documentation: http://packages.python.org/easydev
#
##############################################################################
from importlib import metadata


def get_package_version(package_name):
    try:
        version = metadata.version(package_name)
        return version
    except metadata.PackageNotFoundError:
        return f"{package_name} not found"


version = get_package_version("easydev")

from . import (
    browser,
    chunks,
    codecs,
    config_tools,
    copybutton,
    decorators,
    doc,
    easytest,
    logging_tools,
    misc,
    multicore,
    options,
    paths,
    sphinx_themes,
    tools,
    url,
)
from .browser import browse as onweb
from .chunks import *
from .codecs import *
from .config_tools import *
from .copybutton import *
from .decorators import *

# import dependencies
from .dependencies import get_dependencies
from .doc import *
from .easytest import *
from .logging_tools import *
from .md5tools import md5
from .misc import *
from .multicore import *
from .options import *
from .paths import *
from .profiler import do_profile
from .progressbar import Progress, TextProgressBar, progress_bar
from .sphinx_themes import *
from .timer import Timer
from .tools import *
from .url import *



from easydev import split_into_chunks


def test_chunks():

    assert list(split_into_chunks([0, 1, 2, 3, 4, 5, 6], 2)) == [[0, 2, 4, 6], [1, 3, 5]]
    assert list(split_into_chunks([0, 1, 2, 3, 4, 5], 2)) == [[0, 2, 4], [1, 3, 5]]


test_chunks()



from easydev.decorators import _require, ifpandas, ifpylab, requires


class A(object):
    def __init__(self):
        pass

    def create(self):
        self.a = 1
        self.b = 1

    @requires("a", "what to do")
    def print_str(self):
        print(self.a)

    @requires(["a", "b"], "what to do")
    def print_list(self):
        print(self.a + self.b)

    @_require("a", "what to do")
    def print_list(self):
        print(self.a)


def test():
    a = A()
    try:
        a.print_str()
        assert False
    except:
        assert True
    try:
        a.print_list()
        assert False
    except:
        assert True

    a.create()
    a.print_str()
    a.print_list()


@ifpandas
def test_deco_pandas():
    print(1)


@ifpylab
def test_deco_pylab():
    print(1)



import os

from easydev import CustomConfig
from easydev.config_tools import DynamicConfigParser

from . import test_dir


def test_config_custom():
    c = CustomConfig("dummy")
    c.init()
    c.user_config_dir
    c.remove()


def test_configExample():

    c = DynamicConfigParser(f"{test_dir}/data/config.ini")
    assert "General" in c.sections()
    assert "GA" in c.sections()
    print(c)
    c.remove_section("General")


def test_ordered_dict_attribute():
    # disabled because incompatible with python 3
    c = DynamicConfigParser()
    c.add_section("GA")
    c.GA.test = 2  # this is an attribute only, not  yet a key
    c.add_option("GA", "test", 1)
    c.GA.test = 2
    c["GA"]["test"] = 4
    del c["GA"]["test"]


def test_DynamicConfig(tmpdir):

    outname = tmpdir.join("test.ini")

    # constructor with filename
    dc = DynamicConfigParser(f"{test_dir}/data/config.ini")

    # constructor with existing constructor
    dc = DynamicConfigParser(dc)

    dc.GA
    dc.add_option("GA", "bool", "True")
    dc.get_options("GA")

    # save file and read back
    outname = tmpdir.join("test.ini")
    dc.save(outname)
    dc.save(outname)  # called twice on purpose

    dc.remove_section("GA")
    assert "GA" not in dc.sections()
    print(dc)

    # try something stupid
    try:
        dc = DynamicConfigParser(234)
        assert False
    except TypeError:
        assert True

    dc = DynamicConfigParser()
    try:
        dc.read("test_dummy")
        assert False
    except:
        assert True


def test_DynamicConfigDelete():

    cfg = DynamicConfigParser(f"{test_dir}/data/config.ini")

    dcp = DynamicConfigParser(cfg)
    try:
        del dcp["GA"]
        assert dcp.sections() == ["General"]
    except:
        pass


def test_DynamicConfig_setter(tmpdir):
    outname = tmpdir.join("test.ini")

    dc = DynamicConfigParser()
    dc.add_section("GA")
    dc.add_option("GA", "popsize", 1)
    dc.add_section("General")
    dc.add_option("General", "verbose", True)
    dc.add_option("General", "tag", "test")
    dc.save(outname)

    dc2 = DynamicConfigParser(f"{test_dir}/data/config.ini")
    assert dc == dc2

    dc.GA.test = 10


def test_section2dict():
    dc = DynamicConfigParser()
    dc.add_section("GA")
    dc.add_option("GA", "test", 1)


def test_compare():

    dc = DynamicConfigParser()
    dc.add_section("GA")
    dc.add_option("GA", "test", 1)

    dc2 = DynamicConfigParser()
    dc2.add_section("GA2")
    dc2.add_option("GA2", "test", 1)

    assert (dc == dc2) == False

    dc = DynamicConfigParser()
    dc.add_section("GA")
    dc.add_option("GA", "test", 1)

    dc2 = DynamicConfigParser()
    dc2.add_section("GA")
    dc2.add_option("GA", "test", 2)

    assert (dc == dc2) == False

    dc = DynamicConfigParser()
    dc.add_section("GA")
    dc.add_option("GA", "test", 1)

    dc2 = DynamicConfigParser()
    dc2.add_section("GA")
    dc2.add_option("GA", "test", 1)

    assert (dc == dc2) == True



import os
import sys

from platformdirs import (
    site_config_dir,
    site_data_dir,
    user_cache_dir,
    user_config_dir,
    user_data_dir,
    user_log_dir,
)


class AppDirs(object):
    """Convenience wrapper for getting application dirs."""

    def __init__(self, appname, appauthor=None, version=None, roaming=False, multipath=False):
        self.appname = appname
        self.appauthor = appauthor
        self.version = version
        self.roaming = roaming
        self.multipath = multipath

    @property
    def user_data_dir(self):
        return user_data_dir(self.appname, self.appauthor, version=self.version, roaming=self.roaming)

    @property
    def site_data_dir(self):
        return site_data_dir(self.appname, self.appauthor, version=self.version, multipath=self.multipath)

    @property
    def user_config_dir(self):
        return user_config_dir(self.appname, self.appauthor, version=self.version, roaming=self.roaming)

    @property
    def site_config_dir(self):
        return site_config_dir(self.appname, self.appauthor, version=self.version, multipath=self.multipath)

    @property
    def user_cache_dir(self):
        return user_cache_dir(self.appname, self.appauthor, version=self.version)

    @property
    def user_log_dir(self):
        return user_log_dir(self.appname, self.appauthor, version=self.version)



# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of the easydev software
#  It is a modified version of console.py from the sphinx software
#
#  Copyright (c) 2011-2024
#
#  File author(s): Thomas Cokelaer <cokelaer@gmail.com>
#
#  Distributed under the BSD3 License.
#
#  Website: https://github.com/cokelaer/easydev
#  Documentation: http://packages.python.org/easydev
#
##############################################################################
"""Format colored consoled output. Modified from sphinx.util.console"""
import os
import platform
import sys

__all__ = ["color_terminal", "get_terminal_width", "term_width_line"]


try:
    plf = platform.platform.lower()
    if plf.starstwith("win"):  # pragma: no cover
        import colorama

        colorama.init()
except AttributeError:
    pass

# colors and other functions from the attributes codes are added dynamically to
# this __all__ variable
codes = {}


def get_terminal_width():
    """Returns the current terminal width"""
    try:
        import fcntl
        import struct
        import termios

        call = fcntl.ioctl(0, termios.TIOCGWINSZ, struct.pack("hhhh", 0, 0, 0, 0))
        _, width = struct.unpack("hhhh", call)[:2]
        terminal_width = width
    except (SystemExit, KeyboardInterrupt):  # pragma: no cover
        raise
    except:
        # FALLBACK
        terminal_width = int(os.environ.get("COLUMNS", 80)) - 1
    return terminal_width


def term_width_line(text):
    """prints pruned version of the input text (limited to terminal width)

    :param str text:
    :return str text:
    """
    _tw = get_terminal_width()
    if not codes:
        # if no coloring, don't output fancy backspaces
        return text + "\n"
    else:
        return text.ljust(_tw) + "\r"


def color_terminal():
    """Does terminal allows coloring

    :return: boolean"""
    if not hasattr(sys.stdout, "isatty"):
        return False
    if not sys.stdout.isatty():
        return False
    if "COLORTERM" in os.environ:  # pragma: no cover
        return True
    term = os.environ.get("TERM", "dumb").lower()
    if term in ("xterm", "linux") or "color" in term:
        return True
    return False


def __nocolor():  # pragma: no cover
    """set color codes off"""
    codes.clear()


def __coloron():  # pragma: no cover
    """Set color codes on"""
    codes.update(_orig_codes)


def _colorize(name, text):
    return codes.get(name, "") + text + codes.get("reset", "")


def _create_color_func(name):
    def inner(text):
        return _colorize(name, text)

    globals()[name] = inner


_attrs = {
    "reset": "39;49;00m",
    "bold": "01m",
    "faint": "02m",
    "standout": "03m",
    "underline": "04m",
    "blink": "05m",
}

for _name, _value in _attrs.items():
    codes[_name] = "\x1b[" + _value

_colors = [
    ("black", "darkgray"),
    ("darkred", "red"),
    ("darkgreen", "green"),
    ("brown", "yellow"),
    ("darkblue", "blue"),
    ("purple", "fuchsia"),
    ("turquoise", "teal"),
    ("lightgray", "white"),
]

for i, (dark, light) in enumerate(_colors):
    codes[dark] = "\x1b[%im" % (i + 30)
    codes[light] = "\x1b[%i;01m" % (i + 30)

_orig_codes = codes.copy()

for _name in codes:
    _create_color_func(_name)

# dynamically set the colors
for x in codes.keys():
    __all__.append(x)



from easydev import console
from easydev.console import *


def test_get_terminal_width():
    get_terminal_width()


def test_term_width_line():
    term_width_line("text")


def test_color_terminal():
    color_terminal()


def test_print_color():
    print(purple("\t%s" % "test"))
    print(red("\t%s" % "test"))



# -*- python -*-
#
#  This file is part of easydev software
#
#  Copyright (c) 2011-2024
#
#  File author(s): Thomas Cokelaer <cokelaer@gmail.com>
#
#  Distributed under the BSD3 License.
#
#  Website: https://github.com/cokelaer/easydev
#
##############################################################################
# import logging
import colorlog

__all__ = ["Logging"]


colors = {
    "DEBUG": "cyan",
    "INFO": "green",
    "WARNING": "yellow",
    "ERROR": "red",
    "CRITICAL": "bold_red",
}


class Logging(object):
    """logging utility.

    ::

        >>> l = Logging("root", "INFO")
        >>> l.info("test")
        INFO:root:test
        >>> l.level = "WARNING"
        >>> l.info("test")

    """

    def __init__(self, name="root", level="WARNING", text_color="blue"):
        self._name = name
        self.formatter = colorlog.ColoredFormatter(
            "%(log_color)s%(levelname)-8s[%(name)s:%(lineno)d]: %(reset)s %({})s%(message)s".format(text_color),
            datefmt=None,
            reset=True,
            log_colors=colors,
            secondary_log_colors={},
            style="%",
        )
        self._set_name(name)

        logger = colorlog.getLogger(self._name)
        handler = colorlog.StreamHandler()
        handler.setFormatter(self.formatter)
        logger.addHandler(handler)

    def _set_name(self, name):
        level = self.level
        self._name = name
        logger = colorlog.getLogger(self._name)
        if level == 0:
            self._set_level("WARNING")
        else:
            self._set_level(level)

    def _get_name(self):
        return self._name

    name = property(_get_name, _set_name)

    def _set_level(self, level):
        if isinstance(level, bool):
            if level is True:
                level = "INFO"
            if level is False:
                level = "ERROR"
        if level == 10:
            level = "DEBUG"
        if level == 20:
            level = "INFO"
        if level == 30:
            level = "WARNING"
        if level == 40:
            level = "ERROR"
        if level == 50:
            level = "CRITICAL"
        colorlog.getLogger(self.name).setLevel(level)

    def _get_level(self):
        level = colorlog.getLogger(self.name).level
        if level == 10:
            return "DEBUG"
        elif level == 20:
            return "INFO"
        elif level == 30:
            return "WARNING"
        elif level == 40:
            return "ERROR"
        elif level == 50:
            return "CRITICAL"
        else:
            return level

    level = property(_get_level, _set_level)

    def debug(self, msg):
        colorlog.getLogger(self.name).debug(msg)

    def info(self, msg):
        colorlog.getLogger(self.name).info(msg)

    def warning(self, msg):
        colorlog.getLogger(self.name).warning(msg)

    def critical(self, msg):
        colorlog.getLogger(self.name).critical(msg)

    def error(self, msg):
        colorlog.getLogger(self.name).error(msg)



import os.path

# from unittest.mock import patch
import subprocess

from easydev.misc import cmd_exists, get_home, in_ipynb


def test_get_home():
    get_home()

    # with patch.object(os.path, 'expanduser') as mymock:
    #    mymock.side_effect = ImportError
    #    get_home()


def test_cmd_exists():
    assert cmd_exists("dummy_dummy") == False
    assert cmd_exists("ls") == True

    # with patch.object(subprocess, 'call') as mymock:
    #    mymock.side_effect = Exception
    #    cmd_exists('ls')


def test_in_ipynb():
    assert in_ipynb() == False



from easydev import TempFile
from easydev import tools as tools2
from easydev.tools import *


def test_check_range():
    check_range(1, 0, 1)
    check_range(0, 0, 1)
    check_range(0.5, 0, 1)

    try:
        check_range(1, 0, 1, strict=True)
        assert False
    except:
        assert True
    try:
        check_range(0, 0, 1, strict=True)
        assert False
    except:
        assert True

    try:
        check_range(10, 0, 1, strict=False)
        assert False
    except:
        assert True
    try:
        check_range(-10, 0, 1, strict=False)
        assert False
    except:
        assert True


def test_swapdict():
    assert {1: "a"} == swapdict({"a": 1})

    # if the are non-unique values, we can catch the error or no:
    try:
        swapdict({"a": 1, "b": 1})
        assert False
    except:
        assert True
    swapdict({"a": 1, "b": 1}, check_ambiguity=False)


def test_tools():
    shellcmd("ls")
    shellcmd("ls", show=False)
    shellcmd("ls", show=True)
    output = shellcmd("ls", verbose=True)
    shellcmd("lssssssss", verbose=True, ignore_errors=True)

    execute("ls")


def test_tools2():
    try:
        shellcmd("lsss", verbose=False)
        assert False
    except:
        assert True


def test_checkParams():
    checkParam(1, [1, 2])
    try:
        checkParam(0, [1, 2])
        assert False
    except:
        assert True

    try:
        checkParam(0, 0)
        assert False
    except TypeError:
        assert True


def test_check_param_in_list():
    check_param_in_list(1, [0, 1, 5], "test")
    try:
        check_param_in_list(10, [0, 1, 5])
        assert False
    except:
        assert True
    try:
        check_param_in_list(10, [0, 1, 5], "testt")
        assert False
    except:
        assert True


def test_precision():
    assert tools2.precision(2.123) == 2.12
    assert tools2.precision(2.123, 1) == 2.1
    assert tools2.precision(2.123, 3) == 2.123
    assert tools2.precision(2123, -2) == 2100


def test_attrdict():

    a = AttrDict(value=1)
    assert a.value == 1
    assert "value" in list(a.keys())
    assert 1 in (a.values())

    a.description = "test"
    assert a["description"] == "test"

    a["output"] = "txt"
    assert a.output == "txt"

    d = {"a": {"b": 1}, "aa": 2}
    ad = AttrDict(**d)
    assert ad.a.b == 1
    ad.a.b = 2
    assert ad.a.b == 2

    ad["d"] = 4
    assert ad.d == 4

    try:
        ad.update(1)
        assert False
    except:
        assert True

    # check json capabilities
    fh = TempFile()
    js = ad.to_json()
    ad.to_json(filename=fh.name)
    ad.from_json(fh.name)
    fh.delete()


def test_devtools():
    d = DevTools()
    d.check_param_in_list(1, [1, 2])
    d.check_range(1, 0, 2)
    assert d.list2string(["a", "b"]) == "a,b"
    assert d.swapdict({"a": 1}) == {1: "a"}
    d.to_json({"a": 1})
    assert d.to_list("a") == ["a"]
    import os
    import tempfile

    d.mkdirs(tempfile.mkdtemp() + os.sep + "test")
    try:
        d.check_exists("ttttttttttt")
        assert False
    except:
        assert True
    d.mkdir(tempfile.mkdtemp())


def test_mkdirs():
    import os
    import tempfile

    mkdirs(tempfile.mkdtemp() + os.sep + "test")
    try:
        mkdirs(tempfile.mkdtemp() + os.sep + "test")
        assert False
    except:
        assert True

    # without / , was not working but is now part of the API
    mkdirs(tempfile.mkdtemp())


def test_touch():
    with TempFile() as fh:
        fh.name
        touch(fh.name)



from easydev import browser

from unittest.mock import patch


def test_browse(mocker):
    def func(*args, **kwargs):
        pass

    with patch("webbrowser.open", func):
        with patch("webbrowser.open_new", func):
            browser.browse("http://pypi.python.org", verbose=True)
            browser.browse("pypi.python.org", verbose=True)
            browser.browse(".", verbose=True)


def test_browse_module(mocker):
    from easydev.browser import main

    def func(*args, **kwargs):
        pass

    with patch("webbrowser.open", func):
        main(["browse", "--help"])
        main(["browse", "."])
        main(["browse", "http://www.uniprot.org"])
